@@ -1,4 +1,4 @@
-"""Functions for images."""
+"""Functions for n-dimensional images."""
 import numpy as np
 from tqdm import trange
 from tqdm import tqdm
@@ -35,7 +35,7 @@ def make_slice(n, axis=0, ind=0):
     Parameters
     ----------
     n : int
-        The length of the slice index. 
+        The number of elements in the slice index array.
     axis : list[int]
         The sliced axes.
     ind : list[int] or list[tuple]
@@ -47,11 +47,19 @@ def make_slice(n, axis=0, ind=0):
     idx : tuple
         The slice index array.
     """
+    # Make list if only one axis provided.
     if type(axis) is int:
         axis = [axis]
+        # Can also provide only one axis but provide a tuple for ind, which
+        # selects a range along that axis.
+        if type(ind) is tuple:
+            ind = [ind]
+    # Make list if only one ind provided.
     if type(ind) is int:
         ind = [ind]
+    # Initialize the slice index to select all elements.
     idx = n * [slice(None)]
+    # If any indices were provided, add them to `idx`.
     for k, i in zip(axis, ind):
         if i is None:
             continue
@@ -59,6 +67,7 @@ def make_slice(n, axis=0, ind=0):
             idx[k] = slice(i[0], i[1])
         else:
             idx[k] = i
+    # Must return a tuple.
     return tuple(idx)
 
 
@@ -178,8 +187,22 @@ def project2d_contour(f, axis=(0, 1), level=0.1, shell=None, fpr=None, normalize
 def copy_into_new_dim(f, shape, axis=-1, method='broadcast', copy=False):
     """Copy image into one or more new dimensions.
     
-    The 'broadcast' method is faster since it uses views instead of copies. 
     See 'https://stackoverflow.com/questions/32171917/how-to-copy-a-2d-array-into-a-3rd-dimension-n-times'
+    
+    Parameters
+    ----------
+    f : ndarray
+        An n-dimensional image.
+    shape : n-tuple of ints
+        The shape of the new dimensions.
+    axis : int (0 or -1)
+        If 0, the new dimensions will be inserted before the first axis. If -1, 
+        the new dimensions will be inserted after the last axis. I think
+        values other than 0 or -1 should work; this does not currently 
+        work for `method='broadcast'`.
+    method : {'repeat', 'broadcast'}
+        Whether to use `np.repeat` or `np.expand_dims` and `np.broadcast_to`. The
+        'broadcast' method is faster.
     """
     if type(shape) in [int, np.int32, np.int64]:
         shape = (shape,)
