@@ -1,4 +1,5 @@
 """Plotting routines for phase space distributions."""
+from ipywidgets import interact
 from ipywidgets import interactive
 from ipywidgets import widgets
 from matplotlib import pyplot as plt
@@ -6,6 +7,7 @@ import numpy as np
 import proplot as pplt
 from scipy import optimize as opt
 
+from . import bunch as psb
 from . import image as psi
 from . import utils
 
@@ -64,8 +66,8 @@ def auto_limits(X, sigma=None, **kws):
     
     Parameters
     ----------
-    X : ndarray, shape (n, d)
-        Coordinate array for n points in d-dimensional space.
+    X : ndarray, shape (k, n)
+        Coordinate array for k points in n-dimensional space.
     sigma : float
         If a number is provided, it is used to set the limits relative to 
         the standard deviation of the distribution.
@@ -123,7 +125,7 @@ def plot_profile(
     Parameters
     ----------
     f : ndarray
-        A 2D image.
+        A two-dimensional image.
     xcoords, ycoords : list
         Coordinates of pixel centers.
     ax : matplotlib.pyplt.Axes
@@ -187,7 +189,7 @@ def plot_image(
     Parameters
     ----------
     f : ndarray
-        A 2D image.
+        A two-dimensional image.
     x, y : list
         Coordinates of pixel centers.
     ax : matplotlib.pyplt.Axes
@@ -275,6 +277,7 @@ def _setup_corner(n, diag, labels, limits=None, **fig_kws):
     if labels is None:
         labels = n * [""]
     nrows = ncols = n if diag else n - 1
+    start = 1 if diag else 0
     fig_kws.setdefault("figwidth", 1.5 * nrows)
     fig_kws.setdefault("aligny", True)
     fig, axes = pplt.subplots(
@@ -303,8 +306,8 @@ def _setup_corner(n, diag, labels, limits=None, **fig_kws):
         for i in range(n):
             axes[i, i].format(yspineloc="neither")
     if limits is not None:
-        for i in range(ncols):
-            axes[:, i].format(xlim=limits[i])
+        for j in range(ncols):
+            axes[:, j].format(xlim=limits[j])
         for i in range(start, nrows):
             axes[i, :].format(ylim=limits[i])
     axes.format(xtickminor=True, ytickminor=True, xlocator=('maxn', 3), ylocator=('maxn', 3))
@@ -329,13 +332,13 @@ def corner(
     return_mesh=False,
     **plot_kws,
 ):
-    """Plot 1D/2D projections in a corner plot.
+    """Plot one- and two-dimensional projections in a corner plot.
 
     Parameters
     ----------
     data : ndarray
-        If `data.ndim == 2`, we have a list of N points in D-dimensional space;
-        otherwise, we have a D-dimensional image, i.e., density array.
+        If `data.ndim == 2`, we have the coordinates of k points in n-dimensional 
+        space; otherwise, we have an n-dimensional image.
     kind : {'hist', 'scatter'}
         The kind of 2D plot to make if `data` is a list of points. 
             'hist': 2D histogram
@@ -384,12 +387,14 @@ def corner(
     axes : proplot.gridspec
         Array of subplot axes.
     """
-    # Figure out if data is discrete points or N-D image
+    # Determine whether data is point cloud or image.
     n = data.ndim
     pts = False
     if n == 2:
         n = data.shape[1]
         pts = True
+        
+    # Parse arguments
     diag = diag_kind in ["line", "bar", "step"]
     start = 1 if diag else 0
     if diag_kws is None:
@@ -409,12 +414,17 @@ def corner(
         plot_kws.setdefault("ec", "None")
 
     # Create the figure.
+    if (not pts) and (coords is None):
+        coords = [np.arange(s) for s in data.shape]
     if autolim_kws is None:
         autolim_kws = dict()
     if fig_kws is None:
         fig_kws = dict()
-    if limits is None and pts:
-        limits = auto_limits(data, **autolim_kws)
+    if limits is None:
+        if pts:
+            limits = auto_limits(data, **autolim_kws)
+        else:
+            limits = [(c[0], c[-1]) for c in coords]
     fig, axes = _setup_corner(n, diag, labels, limits, **fig_kws)
 
     # Discrete points
@@ -466,10 +476,8 @@ def corner(
                         prof_kws=prof_kws,
                         **plot_kws,
                     )
-    # Multi-dimensional image
+    # Multidimensional image
     else:
-        if coords is None:
-            coords = [np.arange(s) for s in data.shape]
         # Bivariate plots
         for ii, i in enumerate(range(start, axes.shape[0])):
             for j in range(ii + 1):
@@ -495,12 +503,16 @@ def corner(
         # Univariate plots
         if diag:
             for i in range(n):
-                h = psi.project(data, i)
-                plot1d(coords[i], h / np.max(h), ax=axes[i, i], kind=diag_kind, **diag_kws)
+                profile = psi.project(data, i)
+                profile = profile / np.max(profile)
+                if 'fill_value' in plot_kws:
+                    profile = np.ma.filled(profile, fill_value=plot_kws['fill_value'])
+                plot1d(coords[i], profile, ax=axes[i, i], kind=diag_kind, **diag_kws)                
     # Modify diagonal y axis limits.
     if diag:
         for i in range(n):
             axes[i, i].set_ylim(0, 1.0 / diag_height_frac)
+    # Return items
     if return_fig:
         if return_mesh:
             return fig, axes, mesh
@@ -588,11 +600,12 @@ def slice_matrix(
     debug=False,
     **plot_kws
 ):
-    """Matrix of 2D images as two other dimensions are sliced.
+    """Matrix of 2D projections as two other dimensions are sliced.
     
     In the following, assume `axis_slice`=(0, 1) and `axis_view=(2, 3)`:
     
-    First, `f` is sliced using `ncols` evenly spaced indices along axis 0 and
+    First, `f` is projected onto the (0, 1, 2, 3) axes. The remaining 4D
+    array is sliced using `ncols` evenly spaced indices along axis 0 and
     `nrows` evenly spaced indices along axis 1. The resulting array has shape
     (`ncols`, `nrows`, `f.shape[2]`, `f.shape[3]`). For i in range(`ncols`) and
     j in range(`nrows`), we plot the 2D image `f[i, j, :, :]`. This is done in 
@@ -606,13 +619,11 @@ def slice_matrix(
     
     Fourth, `f` is projected onto axis (2, 3) and plotted in the lower-right p
     panel.
-    
-    ...that was probably not a great explanation.
-    
+        
     Parameters
     ----------
     f : ndarray
-        A four-dimensional image.
+        An n-dimensional image (n >= 4).
     axis_view : 2-tuple of int
         The dimensions to plot.
     axis_slice : 2-tuple of int
@@ -796,14 +807,13 @@ def interactive_proj2d(
     Parameters
     ----------
     f : ndarray
-        An n-dimensional array.
+        An n-dimensional image.
     coords : list[ndarray]
         Coordinate arrays along each dimension. A square grid is assumed.
     default_ind : (i, j)
         Default x and y index to plot.
     slice_type : {'int', 'range'}
         Whether to slice one index along the axis or a range of indices. 
-        (The latest version of `ipywidgets` lets you drag range sliders.)
     dims, units : list[str], shape (n,)
         Dimension names and units.
     prof_kws : dict
@@ -813,13 +823,12 @@ def interactive_proj2d(
     
     Returns
     -------
-    gui : ipywidgets.widgets.interaction.interactive
+    ipywidgets.widgets.interaction.interactive
         This widget can be displayed by calling `IPython.display.display(gui)`. 
     """
     n = f.ndim
     if coords is None:
         coords = [np.arange(f.shape[k]) for k in range(n)]
-
     if dims is None:
         dims = [f"x{i + 1}" for i in range(n)]
     if units is None:
@@ -840,46 +849,32 @@ def interactive_proj2d(
 
     # Widgets
     cmap = widgets.Dropdown(options=cmaps, description="cmap")
+    thresh_checkbox = widgets.Checkbox(value=True, description='thresh')
     thresh = widgets.FloatSlider(
-        value=-8.0,
+        value=-3.3,
         min=-8.0,
         max=0.0,
         step=0.1,
-        description="thresh",
+        description="thresh (log)",
         continuous_update=True,
     )
     discrete = widgets.Checkbox(value=False, description="discrete")
     log = widgets.Checkbox(value=False, description="log")
     contour = widgets.Checkbox(value=False, description="contour")
     profiles = widgets.Checkbox(value=False, description="profiles")
-    scale = widgets.FloatSlider(
-        value=0.15,
-        min=0.0,
-        max=1.0,
-        step=0.01,
-        description="scale",
-        continuous_update=True,
-    )
     dim1 = widgets.Dropdown(options=dims, index=default_ind[0], description="dim 1")
     dim2 = widgets.Dropdown(options=dims, index=default_ind[1], description="dim 2")
-    vmax = widgets.FloatSlider(
-        value=1.0,
-        min=0.0,
-        max=1.0,
-        step=0.01,
-        description="vmax",
-        continuous_update=True,
-    )
-    fix_vmax = widgets.Checkbox(value=False, description="fix vmax")
 
-    # Sliders
+    # Sliders and checkboxes (for slicing). Each unplotted dimension has a 
+    # checkbox which determine if that dimension is sliced. The slice
+    # indices are determined by the slider.
     sliders, checks = [], []
     for k in range(n):
         if slice_type == "int":
             slider = widgets.IntSlider(
                 min=0,
                 max=f.shape[k],
-                value=f.shape[k] // 2,
+                value=(f.shape[k] // 2),
                 description=dims[k],
                 continuous_update=True,
             )
@@ -892,13 +887,13 @@ def interactive_proj2d(
                 continuous_update=True,
             )
         else:
-            raise ValueError("Invalid `slice_type`.")
+            raise ValueError("`slice_type` must be 'int' or 'range'.")
         slider.layout.display = "none"
         sliders.append(slider)
         checks.append(widgets.Checkbox(description=f"slice {dims[k]}"))
 
-    # Hide/show sliders.
     def hide(button):
+        """Hide/show sliders."""
         for k in range(n):
             # Hide elements for dimensions being plotted.
             valid = dims[k] not in (dim1.value, dim2.value)
@@ -911,225 +906,75 @@ def interactive_proj2d(
             # Make sliders respond to check boxes.
             if not checks[k].value:
                 sliders[k].layout.display = "none"
-        # Hide vmax slider if fix_vmax checkbox is not checked.
-        vmax.layout.display = None if fix_vmax.value else "none"
+        # Hide other sliders based on checkboxes.
+        thresh.layout.display = None if thresh_checkbox.value else "none"
 
-    for element in (dim1, dim2, *checks, fix_vmax):
+    # Update the slider list automatically.
+    for element in (dim1, dim2, *checks, thresh_checkbox):
         element.observe(hide, names="value")
     # Initial hide
     for k in range(n):
         if k in default_ind:
             checks[k].layout.display = "none"
             sliders[k].layout.display = "none"
-    vmax.layout.display = "none"
 
-    # I don't know how else to do this.
-    def _update3(
-        cmap,
-        log,
-        profiles,
-        fix_vmax,
-        vmax,
-        dim1,
-        dim2,
-        check1,
-        check2,
-        check3,
-        slider1,
-        slider2,
-        slider3,
-        thresh,
-    ):
-        checks = [check1, check2, check3]
-        sliders = [slider1, slider2, slider3]
+    def update(**kws):
+        """Update the figure."""
+        dim1, dim2 = kws['dim1'], kws['dim2']
+        ind, checks = [], []
+        for i in range(100):
+            if f'check{i}' in kws:
+                checks.append(kws[f'check{i}'])
+            if f'slider{i}' in kws:
+                ind.append(kws[f'slider{i}'])
+        # Return nothing if input does not make sense.
         for dim, check in zip(dims, checks):
             if check and dim in (dim1, dim2):
                 return
-        return _plot_figure(
-            dim1,
-            dim2,
-            checks,
-            sliders,
-            log,
-            profiles,
-            thresh,
-            cmap,
-            fix_vmax,
-            vmax,
-        )
-
-    def _update4(
-        cmap,
-        log,
-        profiles,
-        fix_vmax,
-        vmax,
-        dim1,
-        dim2,
-        check1,
-        check2,
-        check3,
-        check4,
-        slider1,
-        slider2,
-        slider3,
-        slider4,
-        thresh,
-    ):
-        checks = [check1, check2, check3, check4]
-        sliders = [slider1, slider2, slider3, slider4]
-        for dim, check in zip(dims, checks):
-            if check and dim in (dim1, dim2):
-                return
-        return _plot_figure(
-            dim1,
-            dim2,
-            checks,
-            sliders,
-            log,
-            profiles,
-            thresh,
-            cmap,
-            fix_vmax,
-            vmax,
-        )
-
-    def _update5(
-        cmap,
-        log,
-        profiles,
-        fix_vmax,
-        vmax,
-        dim1,
-        dim2,
-        check1,
-        check2,
-        check3,
-        check4,
-        check5,
-        slider1,
-        slider2,
-        slider3,
-        slider4,
-        slider5,
-        thresh,
-    ):
-        checks = [check1, check2, check3, check4, check5]
-        sliders = [slider1, slider2, slider3, slider4, slider5]
-        for dim, check in zip(dims, checks):
-            if check and dim in (dim1, dim2):
-                return
-        return _plot_figure(
-            dim1,
-            dim2,
-            checks,
-            sliders,
-            log,
-            profiles,
-            thresh,
-            cmap,
-            fix_vmax,
-            vmax,
-        )
-
-    def _update6(
-        cmap,
-        log,
-        profiles,
-        fix_vmax,
-        vmax,
-        dim1,
-        dim2,
-        check1,
-        check2,
-        check3,
-        check4,
-        check5,
-        check6,
-        slider1,
-        slider2,
-        slider3,
-        slider4,
-        slider5,
-        slider6,
-        thresh,
-    ):
-        checks = [check1, check2, check3, check4, check5, check6]
-        sliders = [slider1, slider2, slider3, slider4, slider5, slider6]
-        for dim, check in zip(dims, checks):
-            if check and dim in (dim1, dim2):
-                return
-        return _plot_figure(
-            dim1,
-            dim2,
-            checks,
-            sliders,
-            log,
-            profiles,
-            thresh,
-            cmap,
-            fix_vmax,
-            vmax,
-        )
-
-    update = {3: _update3, 4: _update4, 5: _update5, 6: _update6,}[n]
-
-    def _plot_figure(
-        dim1,
-        dim2,
-        checks,
-        sliders,
-        log,
-        profiles,
-        thresh,
-        cmap,
-        fix_vmax,
-        vmax,
-    ):
         if dim1 == dim2:
             return
+        # Slice and project the distribution.
         axis_view = [dims.index(dim) for dim in (dim1, dim2)]
         axis_slice = [dims.index(dim) for dim, check in zip(dims, checks) if check]
-        ind = sliders
         for k in range(n):
             if type(ind[k]) is int:
                 ind[k] = (ind[k], ind[k] + 1)
         ind = [ind[k] for k in axis_slice]
-        H = f[psi.make_slice(f.ndim, axis_slice, ind)]
-        H = psi.project(H, axis_view)
-        H = np.ma.masked_less_equal(H, 10.0 ** thresh)
-        plot_kws.update(
-            {
-                "profx": profiles,
-                "profy": profiles,
-                "cmap": cmap,
-                "thresh": 10.0 ** thresh,
-                "thresh_type": "frac",
-                "norm": "log" if log else None,
-                "vmax": vmax if fix_vmax else None,
-                "fill_value": 0,
-            }
-        )
+        idx = psi.make_slice(f.ndim, axis_slice, ind)
+        _f = psi.project(f[idx], axis_view)
+        # Update plotting key word arguments.
+        plot_kws['cmap'] = kws['cmap']
+        plot_kws['fill_value'] = 0
+        plot_kws['norm'] = 'log' if kws['log'] else None
+        plot_kws['profx'] = plot_kws['profy'] = kws['profiles']
+        plot_kws['thresh'] = (10.0**kws['thresh']) if kws['thresh_checkbox'] else None
+        # Plot the projection onto the specified axes. 
         fig, ax = pplt.subplots()
-        plot_image(H, x=coords[axis_view[0]], y=coords[axis_view[1]], ax=ax, **plot_kws)
+        plot_image(
+            _f,
+            x=coords[axis_view[0]], 
+            y=coords[axis_view[1]], 
+            ax=ax, 
+            **plot_kws
+        )
         ax.format(xlabel=dims_units[axis_view[0]], ylabel=dims_units[axis_view[1]])
         plt.show()
-
-    kws = dict()
-    kws["dim1"] = dim1
-    kws["dim2"] = dim2
+        
+    # Pass key word arguments for `update`.
+    kws = {
+        'cmap': cmap,
+        'log': log,
+        'profiles': profiles,
+        'thresh_checkbox': thresh_checkbox,
+        'thresh': thresh,
+        'dim1': dim1,
+        'dim2': dim2,
+    }
     for i, check in enumerate(checks, start=1):
-        kws[f"check{i}"] = check
+        kws[f'check{i}'] = check
     for i, slider in enumerate(sliders, start=1):
-        kws[f"slider{i}"] = slider
-    kws["log"] = log
-    kws["profiles"] = profiles
-    kws["thresh"] = thresh
-    kws["fix_vmax"] = fix_vmax
-    kws["vmax"] = vmax
-    kws["cmap"] = cmap
-    gui = interactive(update, **kws)
-    return gui
+        kws[f'slider{i}'] = slider  
+    return interactive(update, **kws)
 
 
 def interactive_proj1d(
@@ -1140,6 +985,7 @@ def interactive_proj1d(
     dims=None,
     units=None,
     kind="bar",
+    fig_kws=None,
     **plot_kws,
 ):
     """1D projection of image `f` with interactive slicing.
@@ -1147,18 +993,19 @@ def interactive_proj1d(
     Parameters
     ----------
     f : ndarray
-        An n-dimensional array.
+        An n-dimensional image.
     coords : list[ndarray]
         Grid coordinates for each dimension.
     default_ind : int
         Default index to plot.
     slice_type : {'int', 'range'}
         Whether to slice one index along the axis or a range of indices. 
-        (The latest version of `ipywidgets` lets you drag range sliders.)
     dims, units : list[str], shape (n,)
         Dimension names and units.
     kind : {'bar', 'line'}
         The kind of plot to draw.
+    fig_kws : dict
+        Key word arguments passed to `proplot.subplots`. 
     **plot_kws
         Key word arguments passed to 1D plotting function.
         
@@ -1170,7 +1017,6 @@ def interactive_proj1d(
     n = f.ndim
     if coords is None:
         coords = [np.arange(f.shape[k]) for k in range(n)]
-
     if dims is None:
         dims = [f"x{i + 1}" for i in range(n)]
     if units is None:
@@ -1179,6 +1025,9 @@ def interactive_proj1d(
     for dim, unit in zip(dims, units):
         dims_units.append(f"{dim}" + f" [{unit}]" if unit != "" else dim)
     plot_kws.setdefault("color", "black")
+    if fig_kws is None:
+        fig_kws = dict()
+    fig_kws.setdefault('figsize', (4.5, 1.5))
 
     # Widgets
     dim1 = widgets.Dropdown(options=dims, index=default_ind, description="dim")
@@ -1208,8 +1057,8 @@ def interactive_proj1d(
         sliders.append(slider)
         checks.append(widgets.Checkbox(description=f"slice {dims[k]}"))
 
-    # Hide/show sliders.
     def hide(button):
+        """Hide/show sliders based on checkboxes."""
         for k in range(n):
             # Hide elements for dimensions being plotted.
             valid = dims[k] != dim1.value
@@ -1223,6 +1072,7 @@ def interactive_proj1d(
             if not checks[k].value:
                 sliders[k].layout.display = "none"
 
+    # Update the slider list automatically.
     for element in (dim1, *checks):
         element.observe(hide, names="value")
     # Initial hide
@@ -1230,136 +1080,79 @@ def interactive_proj1d(
         if k == default_ind:
             checks[k].layout.display = "none"
             sliders[k].layout.display = "none"
-
-    # I don't know how else to do this.
-    def _update2(
-        dim1, check1, check2, slider1, slider2,
-    ):
-        checks = [check1, check2]
-        sliders = [slider1, slider2]
+            
+    def update(**kws):
+        """Update the figure."""
+        dim1 = kws['dim1']
+        ind, checks = [], []
+        for i in range(100):
+            if f'check{i}' in kws:
+                checks.append(kws[f'check{i}'])
+            if f'slider{i}' in kws:
+                ind.append(kws[f'slider{i}'])
+        # Return nothing if input does not make sense.
         for dim, check in zip(dims, checks):
             if check and dim == dim1:
                 return
-        return _plot_figure(dim1, checks, sliders)
-
-    def _update3(
-        dim1, check1, check2, check3, slider1, slider2, slider3,
-    ):
-        checks = [check1, check2, check3]
-        sliders = [slider1, slider2, slider3]
-        for dim, check in zip(dims, checks):
-            if check and dim == dim1:
-                return
-        return _plot_figure(dim1, checks, sliders)
-
-    def _update4(
-        dim1, check1, check2, check3, check4, slider1, slider2, slider3, slider4,
-    ):
-        checks = [check1, check2, check3, check4]
-        sliders = [slider1, slider2, slider3, slider4]
-        for dim, check in zip(dims, checks):
-            if check and dim == dim1:
-                return
-        return _plot_figure(dim1, checks, sliders)
-
-    def _update5(
-        dim1,
-        check1,
-        check2,
-        check3,
-        check4,
-        check5,
-        slider1,
-        slider2,
-        slider3,
-        slider4,
-        slider5,
-    ):
-        checks = [check1, check2, check3, check4, check5]
-        sliders = [slider1, slider2, slider3, slider4, slider5]
-        for dim, check in zip(dims, checks):
-            if check and dim == dim1:
-                return
-        return _plot_figure(dim1, checks, sliders)
-
-    def _update6(
-        dim1,
-        check1,
-        check2,
-        check3,
-        check4,
-        check5,
-        check6,
-        slider1,
-        slider2,
-        slider3,
-        slider4,
-        slider5,
-        slider6,
-    ):
-        checks = [check1, check2, check3, check4, check5, check6]
-        sliders = [slider1, slider2, slider3, slider4, slider5, slider6]
-        for dim, check in zip(dims, checks):
-            if check and dim == dim1:
-                return
-        return _plot_figure(dim1, checks, sliders)
-
-    update_dict = {
-        2: _update2,
-        3: _update3,
-        4: _update4,
-        5: _update5,
-        6: _update6,
-    }
-    update = update_dict[n]
-
-    def _plot_figure(dim1, checks, sliders):
+        # Slice, then project onto the specified axis.
         axis_view = dims.index(dim1)
         axis_slice = [dims.index(dim) for dim, check in zip(dims, checks) if check]
-        ind = sliders
         for k in range(n):
             if type(ind[k]) is int:
                 ind[k] = (ind[k], ind[k] + 1)
         ind = [ind[k] for k in axis_slice]
-        _f = f[psi.make_slice(f.ndim, axis_slice, ind)]
-        p = psi.project(_f, axis_view)
-
-        fig, ax = pplt.subplots(figsize=(4.5, 1.5))
+        idx = psi.make_slice(f.ndim, axis_slice, ind)
+        profile = psi.project(f[idx], axis_view)  
+        profile = profile / np.sum(profile)
+        # Plot the projection.
+        fig, ax = pplt.subplots(**fig_kws)
         ax.format(xlabel=dims_units[axis_view])
-        x = coords[axis_view]
-        y = p / np.sum(p)
         if kind == "bar":
-            ax.bar(x, y, **plot_kws)
+            ax.bar(coords[axis_view], profile, **plot_kws)
         elif kind == "line":
-            ax.plot(x, y, **plot_kws)
+            ax.plot(coords[axis_view], profile, **plot_kws)
         elif kind == "step":
-            ax.plot(x, y, drawstyle="steps-mid", **plot_kws)
+            ax.plot(coords[axis_view], profile, drawstyle="steps-mid", **plot_kws)
         plt.show()
 
-    kws = dict()
-    kws["dim1"] = dim1
+    kws = {'dim1': dim1}
     for i, check in enumerate(checks, start=1):
         kws[f"check{i}"] = check
     for i, slider in enumerate(sliders, start=1):
         kws[f"slider{i}"] = slider
-    gui = interactive(update, **kws)
-    return gui
+    return interactive(update, **kws)
 
 
 def interactive_proj2d_discrete(
     X,
     limits=None,
-    nbins=10,
+    nbins=30,
     default_ind=(0, 1),
     slice_type="int",  # {'int', 'range'}
     dims=None,
     units=None,
+    prof_kws=None,
     **plot_kws,
 ):
     """This mirrors `interactive_proj2d` for point clouds.
-    
-    This is useful because we do not have to compute/store a 6D histogram. (It
-    currently works for 6D data, not ND data.)
+        
+    Parameters
+    ----------
+    X : ndarray, shape (k, n)
+        Coordinates of k points in n-dimensional space.
+    limits : list[(min, max)]
+        Limits along each axis.
+    nbins : int
+        Default number of bins for slicing/viewing. Both can be changed with
+        sliders.
+    default_ind : (i, j)
+        Default view axis.
+    slice_type : {'int', 'range'}
+        Whether to slice one index along the axis or a range of indices. 
+    dims, units : list[str], shape (n,)
+        Dimension names and units.
+    **plot_kws
+        Key word arguments passed to `plot_image`.
     """
     n = X.shape[1]
     if limits is None:
@@ -1372,14 +1165,15 @@ def interactive_proj2d_discrete(
     for dim, unit in zip(dims, units):
         dims_units.append(f"{dim}" + f" [{unit}]" if unit != "" else dim)
     plot_kws.setdefault("colorbar", True)
+    plot_kws["prof_kws"] = prof_kws
 
     # Widgets
     nbins_default = nbins
     dim1 = widgets.Dropdown(options=dims, index=default_ind[0], description="dim 1")
     dim2 = widgets.Dropdown(options=dims, index=default_ind[1], description="dim 2")
-    nbins = widgets.IntSlider(min=1, max=100, value=nbins_default, description='grid res')
-    nbins_plot = widgets.IntSlider(min=1, max=100, value=nbins_default, description='plot res')
-    autobin = widgets.Checkbox(description='auto plot res', value=False)
+    nbins = widgets.IntSlider(min=2, max=100, value=nbins_default, description='grid res')
+    nbins_plot = widgets.IntSlider(min=2, max=200, value=nbins_default, description='plot res')
+    autobin = widgets.Checkbox(description='auto plot res', value=True)
     log = widgets.Checkbox(description='log', value=False)
     prof = widgets.Checkbox(description='profiles', value=False)
     sliders, checks = [], []
@@ -1406,8 +1200,8 @@ def interactive_proj2d_discrete(
         sliders.append(slider)
         checks.append(widgets.Checkbox(description=f"slice {dims[k]}"))
 
-    # Hide/show sliders.
     def hide(button):
+        """Hide inactive sliders."""
         for k in range(n):
             # Hide elements for dimensions being plotted.
             valid = dims[k] not in (dim1.value, dim2.value)
@@ -1420,90 +1214,80 @@ def interactive_proj2d_discrete(
             # Make sliders respond to check boxes.
             if not checks[k].value:
                 sliders[k].layout.display = "none"
-                
-    for element in (dim1, dim2, *checks):
+            nbins_plot.layout.display = "none" if autobin.value else None
+               
+    # Make slider visiblity depend on checkmarks.
+    for element in (dim1, dim2, *checks, autobin):
         element.observe(hide, names="value")
         
     # Initial hide
+    nbins_plot.layout.display = "none"
     for k in range(n):
         if k in default_ind:
             checks[k].layout.display = "none"
             sliders[k].layout.display = "none"
-                                                  
-    def update(
-        log,
-        prof,
-        dim1,
-        dim2,
-        check1,
-        check2,
-        check3,
-        check4,
-        check5,
-        check6,
-        slider1,
-        slider2,
-        slider3,
-        slider4,
-        slider5,
-        slider6,
-        nbins,
-        nbins_plot,
-        autobin,
-    ):
-        if dim1 == dim2:
-            return
-        checks = [check1, check2, check3, check4, check5, check6]
-        sliders = [slider1, slider2, slider3, slider4, slider5, slider6]        
+            
+    def update(**kws):
+        dim1 = kws['dim1']
+        dim2 = kws['dim2']
+        nbins = kws['nbins']
+        nbins_plot = kws['nbins_plot']
+        autobin = kws['autobin']
+        ind, checks = [], []
+        for i in range(100):
+            if f'check{i}' in kws:
+                checks.append(kws[f'check{i}'])
+            if f'slider{i}' in kws:
+                _ind = kws[f'slider{i}']
+                if type(_ind) is int:
+                    _ind = (_ind, _ind + 1)
+                ind.append(_ind)
+                
+        # Return nothing if input does not make sense.
         for dim, check in zip(dims, checks):
             if check and dim in (dim1, dim2):
                 return
-        checks = checks[:n]
-        sliders = sliders[:n]
-            
-        # Find particles within box.
+        if dim1 == dim2:
+            return
+                
+        # Slice the distribution
         axis_view = [dims.index(dim) for dim in (dim1, dim2)]
         axis_slice = [dims.index(dim) for dim, check in zip(dims, checks) if check]
-        conditions = []
-        for i in range(n):
-            if i in axis_slice:
-                bin_width = np.abs(limits[i][1] - limits[i][0]) / nbins
-                ind = sliders[i]
-                if type(ind) is int:
-                    ind = (ind, ind + 1)
-                umin = limits[i][0] + ind[0] * bin_width
-                umax = limits[i][0] + ind[1] * bin_width
-            else:
-                umin = -np.inf
-                umax = +np.inf
-            conditions.append(X[:, i] > umin)
-            conditions.append(X[:, i] < umax)
-            idx = np.logical_and.reduce(conditions)
-        
+        edges = [np.linspace(umin, umax, nbins + 1) for (umin, umax) in limits]
+        if axis_slice:
+            center, width = [], []
+            for _axis in axis_slice:
+                _edges = edges[_axis]
+                imin, imax = ind[_axis]
+                width.append(_edges[imax] - _edges[imin])
+                center.append(0.5 * (_edges[imax] + _edges[imin]))
+            Xs = psb.slice_box(X, axis=axis_slice, center=center, width=width)
+        else:
+            Xs = X[:, :]
+            
         # Compute 2D histogram of remaining particles.
-        _X = X[idx]
-        _X = _X[:, axis_view]
         _nbins = 'auto' if autobin else nbins_plot
-        xedges = np.histogram_bin_edges(_X[:, 0], bins=_nbins, range=limits[axis_view[0]])
-        yedges = np.histogram_bin_edges(_X[:, 1], bins=_nbins, range=limits[axis_view[1]])
+        xedges = np.histogram_bin_edges(Xs[:, axis_view[0]], bins=_nbins, range=limits[axis_view[0]])
+        yedges = np.histogram_bin_edges(Xs[:, axis_view[1]], bins=_nbins, range=limits[axis_view[1]])
         edges = [xedges, yedges]
-        im, _ = np.histogramdd(_X, bins=edges)
-        centers = [utils.bin_centers(e) for e in edges]
+        centers = [utils.get_centers(e) for e in edges]
+        image, _, _ = np.histogram2d(Xs[:, axis_view[0]], Xs[:, axis_view[1]], bins=edges)
         
-        # Plot image.
-        plot_kws['norm'] = 'log' if log else None
-        plot_kws['profx'] = plot_kws['profy'] = prof
+        # Update plot key word arguments.
+        plot_kws['norm'] = 'log' if kws['log'] else None
+        plot_kws['profx'] = plot_kws['profy'] = kws['prof']
+        # Plot the image.
         fig, ax = pplt.subplots()
-        plot_image(im, x=centers[0], y=centers[1], ax=ax, **plot_kws)
+        plot_image(image, x=centers[0], y=centers[1], ax=ax, **plot_kws)
         ax.format(xlabel=dims_units[axis_view[0]], ylabel=dims_units[axis_view[1]])
         plt.show()
 
     kws = dict()
-    kws["dim1"] = dim1
-    kws["dim2"] = dim2
     kws["log"] = log
     kws["prof"] = prof
     kws["autobin"] = autobin
+    kws["dim1"] = dim1
+    kws["dim2"] = dim2
     for i, check in enumerate(checks, start=1):
         kws[f"check{i}"] = check
     for i, slider in enumerate(sliders, start=1):
